#ifndef __BuddhaGameComponent_H__
#define __BuddhaGameComponent_H__

#include "BuddhaGameTime.h"

namespace BuddhaXNA
{
	class Game;
	class GameComponentCollection;

	/// SQ[R|[lg̊NX
	class GameComponent
	{
	friend GameComponentCollection;

	private:
		/// XVB̒lR|[lg Update Ăяo܂B
		int mUpdateOrder;

		/// Abvf[gL
		bool mEnable;

	protected:
		/// eGame
		Game* m_pGame;

	public:
		GameComponent(Game *game)
			: m_pGame(game),
			  mUpdateOrder(0),
			  mEnable(true)
		{
		}

		virtual ~GameComponent() {}

		/**
		 * @brief ̃R|[lg
		 * @remarks
		 *      ̃o֐́ÃR|[lg̏KvȂƂɁA Game NXĂт
		 *      ܂B
		 *
		 *      \li Game::Update() ̃[vJnOɁAGame::Initialize()ŒǉꂽR|[l
		 *          ǵAGame::Update() JnOɈxR|[lgRNV GameComponent::Initialize()
		 *          Ăт܂B
		 *      \li C[vJnɃR|[lgRNVɒǉꂽR|[lǵAǉ
		 *          iKł GameComponent::Initialize() Ăт܂B
		 *      \li R|[lgDx̍iUpdateOrderႢjR|[lgǉƁA
		 *          ̂Ԃ񃋁[vAUpdate()Qs܂B XNA ̃oOł͂ȂƎv
		 *          ܂A_ł͂̎dlɂ킹̗pĂ܂B
		 */
		virtual void Initialize() = NULL;

		virtual void Update(GameTime gameTime) {}

		/**
		 * @brief XVZbg܂B̒l̏R|[lg珇Ԃ Update() Ăяo܂B
		 */
		void SetUpdateOrder(int order)
		{
			mUpdateOrder = order;
		}

		/// XV擾
		int GetUpdateOrder()
		{
			return mUpdateOrder;
		}

		/// ̃R|[lg̗L/Zbg
		void SetEnable(bool value)
		{
			mEnable = value;
		}

		/// ̃R|[lgLǂׂ
		bool GetEnable()
		{
			return mEnable;
		}
	};

	/// `掞ɒʒm󂯂 GameComponent
	class DrawableComponent : public GameComponent
	{
		friend GameComponentCollection;

	private:
		/// `揇Bقǐɕ`悳܂B
		int mDrawOrder;

	public:
		DrawableComponent(Game *game)
			: GameComponent(game),
			  mDrawOrder(0)
		{
		}

		virtual ~DrawableComponent() {}

		/**
		 * @brief R|[lgKvƂOtBbNRec[h
		 * @remarks
		 *     ̃o֐́AKvɉ Game NXĂт܂B
		 *     R|[lgRNVɒǉƂ̍ŏ̃R[obN̂
		 *     GameComponent::Initialize() ƓłAK Initialize() ɑČĂт܂B
		 *
		 *     \li Game::Initialize() ̒iKŃRNVɊi[ꂽ DrawableGameComplenent ́A
		 *         Game::Update() ̃[vJnOɁA loadAllContent = true ŁÃo
		 *         Ăт܂B
		 *     \li foCX̕ŁAǗÕOtBbNRecă[hKv
		 *         ꍇA loadAllContent = false ł̃o֐Ăт܂B
		 *     \li ̏ꍇAo֐ Game::LoadGraphicComponent() ̓RNV̂
		 *         o֐ׂČĂтꂽɁAŌɌĂт܂BtȌ͓łB
		 *
		 * @param loadAllContent true ̂ƂAǗ̃OtBbNRec[h
		 */
		virtual void LoadGraphicsContent(bool loadAllContent) {}
		virtual void Draw(GameTime gameTime) {}

		/**
		 * @brief R|[lgĂOtBbNRec
		 * @remarks
		 *     ̃o֐́AKvɉ Game NXĂт܂B
		 *
		 *     \li foCX̏ȂǂɂAxOtBbNRecȂ΂ȂȂꍇA
		 *         unloadAllContent = false ł̊֐Ăт܂B̏ꍇAɃR|[lg
		 *         RNṼA[hsAŌ Game NX̃A[hĂт܂B
		 * 
		 * @todo RNVXgrꂽۂɁAǂ̃^C~OŃA[h邩肵Ď
 		 * @param unloadAllContent true ̂ƂAǗ̃OtBbNRec
		 */
		virtual void UnloadGraphicContent(bool unloadAllContent) {}

		/**
		 * @brief `揇Zbg
		 * @param order V`揇
		 */
		void SetDrawOrder(int order)
		{
			mDrawOrder = order;
		}

		/// `揇擾
		int GetDrawOrder()
		{
			return mDrawOrder;
		}
	};

	/**
	 * GameComponent ̃RNV
	 * @remarks
	 *     ̃RNV͊J҂ɂ钼ڑ󂯂邽߁ASTLReił͂ȂƗ
	 *     NXƂȂĂ܂B
	 * @todo Remove Aēx Add R|[lg Initialize Ă΂Ȃ
	 * @todo Remove Aēx Add R|[lg LoadGraphicContent Ă΂ȂiAfoCX͕ʏj
	 */
	class GameComponentCollection
	{
		friend Game;

	private:
		/**
		 * @todo Ƃ multimap 
		 */
		typedef std::list<GameComponent*> GameComponentList;

		/**
		 * @todo Ƃ multimap 
		 */
		typedef std::list<DrawableComponent*> DrawableComponentList;

		/// C[vn܂Ă邩ǂBGameNXύXB
		bool mHasMainloopBegan;

		GameComponentList mUpdatableList;
		DrawableComponentList mDrawableList;

		GameComponentCollection();

		virtual ~GameComponentCollection();

	public:
		/// R|[lgǉ
		void Add(GameComponent *comp);

		/// R|[lgǉ
		void Add(DrawableComponent *comp);

		/// R|[lg菜
		void Remove(GameComponent *comp);

		/// R|[lg菜
		void Remove(DrawableComponent *comp);

	private:
		/// Update [vs
		void _update(GameTime gameTime);

		/// Draw [vs
		void _draw(GameTime gameTime);

		/**
		 * @internal
		 * @brief ׂĂ̓o^ς݃R|[lgAOtBbN[h
		 * @remarks
		 *     ̃\bh Game R|[lg Game::Initialize() IAC[vJn
		 *     ܂ł̊ԂɌĂяo܂BׂĂ̓o^ς GameComponent  Initialize() ĂсAׂĂ̓o^
		 *     ς DrawableComponent  LoadGraphicContent()  loadAllContent = true ŌĂяo܂B
         *     ̃\bhĂяoƎI mHasMainloopBegan  true ƂȂ܂B
		 */
		void _initializeAll(void);

		/**
		 * @internal
		 * @brief ׂĂ̓o^ς DrawableComponent  LoadGraphicContent() \bhĂяo
		 */
		void _loadGraphicContent(bool loadAllContent);

		/**
		 * @internal
		 * @brief ׂĂ̓o^ς DrawableComponent  UnloadGraphicContent() \bhĂяo
		 */
		void _unloadGraphicContent(bool loadAllContent);
	};
}

#endif