#ifndef __BuddhaVector3_H__
#define __BuddhaVector3_H__

#include <d3d9.h>
#include "BuddhaMath.h"

namespace BuddhaXNA
{
	struct Matrix;
	struct Quaternion;

	/**
	 * @brief RxNg\
	 * @todo ܂Kv
	 * @todo XJ[lɑ΂鉉ZqI[o[[h͕svȂ
	 * @todo NXςȂ...
	 */
	struct Vector3
	{
	public:
		FLOAT X;
		FLOAT Y;
		FLOAT Z;
		//------------------------------------------------------------------------
		inline Vector3& operator = (const Vector3& value)
		{
			X = value.X;
			Y = value.Y;
			Z = value.Z;

			return *this;
		}
		//------------------------------------------------------------------------
		inline Vector3& operator = (const FLOAT value)
		{
			X = value;
			Y = value;
			Z = value;

			return *this;
		}
		//------------------------------------------------------------------------
		inline bool operator == (const Vector3& value)
		{
			return (X == value.X && Y == value.Y && Z == value.Z);
		}
		//------------------------------------------------------------------------
		inline bool operator != (const Vector3& value)
		{
			return (X != value.X || Y != value.Y || Z != value.Z);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator + (const Vector3& value)
		{
			return Vector3(X + value.X, Y + value.Y, Z + value.Z);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator - (const Vector3& value)
		{
			return Vector3(X - value.X, Y - value.Y, Z - value.Z);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator * (const FLOAT value)
		{
			return Vector3(X * value, Y * value, Z * value);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator * (const Vector3 &value)
		{
			return Vector3(X * value.X, Y * value.Y, Z * value.Z);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator / (const Vector3 &value)
		{
			return Vector3(X / value.X, Y / value.Y, Z / value.Z);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator - ()
		{
			return Vector3(-X, -Y, -Z);
		}
		//------------------------------------------------------------------------
		inline Vector3 operator += (const Vector3 &value)
		{
			X += value.X;
			Y += value.Y;
			Z += value.Z;

			return *this;
		}
		//------------------------------------------------------------------------
		inline Vector3 operator -= (const Vector3 &value)
		{
			X -= value.X;
			Y -= value.Y;
			Z -= value.Z;

			return *this;
		}
		//------------------------------------------------------------------------
		inline friend bool operator != (const Vector3 &left, const Vector3& right)
		{
			return (left.X != right.X || left.Y != right.Y || left.Z != right.Z);
		}
		//------------------------------------------------------------------------
		inline friend Vector3 operator * (FLOAT scalar, const Vector3& value)
		{
			return Vector3(scalar * value.X, scalar * value.Y, scalar * value.Z);
		}
		//------------------------------------------------------------------------
		inline friend Vector3 operator * (const Vector3& left, const Vector3& right)
		{
			return Vector3(left.X * right.X, left.Y * right.Y, left.Z * right.Z);
		}
		//------------------------------------------------------------------------
		inline friend Vector3 operator + (const Vector3& left, const Vector3& right)
		{
			return Vector3(left.X + right.X, left.Y + right.Y, left.Z + right.Z);
		}
		//------------------------------------------------------------------------
		inline friend Vector3 operator - (FLOAT scalar, const Vector3& value)
		{
			return Vector3(scalar - value.X, scalar - value.Y, scalar - value.Z);
		}
		//------------------------------------------------------------------------
		inline friend Vector3 operator - (const Vector3& value, FLOAT scalar)
		{
			return Vector3(value.X - scalar, value.Y - scalar, value.Z - scalar);
		}
		//------------------------------------------------------------------------
		inline friend Vector3 operator - (const Vector3& left, const Vector3& right)
		{
			return Vector3(left.X - right.X, left.Y - right.Y, left.Z - right.Z);
		}
		//------------------------------------------------------------------------
		Vector3()
		{
			X = 0;
			Y = 0;
			Z = 0;
		}
		//------------------------------------------------------------------------
		Vector3(const float x, const float y, const float z)
		{
			X = x;
			Y = y;
			Z = z;
		}
		//------------------------------------------------------------------------
		Vector3(const Vector3& value)
		{
			X = value.X;
			Y = value.Y;
			Z = value.Z;
		}

		/// xNg̒ij擾
		inline FLOAT Distance(void)
		{
			return (FLOAT)(Math::Sqrt(X * X + Y * Y + Z * Z));
		}

		/**
		 * @brief xNg̒ij̓𓾂
		 * @remarks
		 *     ̊֐͕ƂȂ̂ Distance() 荂ł
		 */
		inline FLOAT DistanceSquared(void)
		{
			return X * X + Y * Y + Z * Z;
		}

		/// PʃxNg
		void Normalize();

		/// xNgԂ
		static Vector3 Negate(const Vector3& value);

		/// xNgvZ
		static void Negate(const Vector3& in, Vector3& out);

		/// vector1  vector2 ̓ρihbgρjĕԂ
		static FLOAT Dot(const Vector3& vector1, const Vector3& vector2);

		/// vector1  vector2 ̊OρiNXρjԂ
		static Vector3 Cross(const Vector3& vector1, const Vector3& vector2);

		/// ˃xNgԂ
		static Vector3 Reflect(const Vector3& value, const Vector3& normal);

		/// ˃xNg𓾂
		static void Reflect(const Vector3& value, const Vector3& normal, Vector3& out);
		
		/// ^ꂽ}gNXŃxNgό`܂
		static Vector3 Transform(const Vector3& vector, const Matrix& matrix);

		/// w肳ꂽNI[^jI vector ό`l𓾂܂
		static Vector3 Transform(const Vector3 &vector, const Quaternion &q);

		/// w肳ꂽNI[^jI vector ό`A out ɒlԂ܂
		static void Transform(const Vector3 &vector, const Quaternion &q, Vector3 &out);

		/// ^ꂽ}gNXŃxNgό`A out ɒlԂ܂
		static void Transform(const Vector3& vector, const Matrix& matrix, Vector3& out);

		/// D3DVECTOR ^ Vector3 ^̒lRo[g
		static void CreateD3DVECTOR(const Vector3 &in, D3DVECTOR &out);

		static const Vector3 Zero;	///< SlOVector3
		static const Vector3 One;	///< SlPVector3

		static const Vector3 Up;	///< PʃxNg
		static const Vector3 Down;	///< PʃxNg

		static const Vector3 Left;		///< ̒PʃxNg
		static const Vector3 Right;		///< E̒PʃxNg

		static const Vector3 Forward;	///< ʌ̒PʃxNg
		static const Vector3 Backward;	///< ̒PʃxNg

		static const Vector3 UnitX;	///< X̒P
		static const Vector3 UnitY;	///< Y̒P
		static const Vector3 UnitZ;	///< Z̒P
	};
}


#endif