#ifndef __BuddhaVertexBuffer_H__
#define __BuddhaVertexBuffer_H__

#include <d3d9.h>
#include "BuddhaXNAPrerequisites.h"
#include "BuddhaGraphicsDevice.h"
#include "BuddhaVertexDeclaration.h"
#include "BuddhaSharedPtr.h"
#include "BuddhaResourceUsage.h"

namespace BuddhaXNA
{
	struct VertexPositionNormalTexture;
	struct VertexPositionColor;

	class VertexBuffer
	{
	protected:
		LPDIRECT3DVERTEXBUFFER9 m_pBuffer;

	public:
		VertexBuffer(GraphicsDevice * device, UINT length, DWORD usage, D3DPOOL pool);
		VertexBuffer(GraphicsDevice * device, UINT length, DWORD usage, DWORD fvf, D3DPOOL pool);

		void SetData(void *src, size_t size);

		inline void SetData(VertexPositionNormalTexture *src, UINT length)
		{
			SetData((VOID*)src, sizeof(VertexPositionNormalTexture) * length);
		}

		inline void SetData(VertexPositionColor *src, UINT length)
		{
			SetData((VOID*)src, sizeof(VertexPositionColor) * length);
		}

		LPDIRECT3DVERTEXBUFFER9 _getBuffer(void) const
		{
			return m_pBuffer;
		}

		virtual ~VertexBuffer()
		{
			SAFE_RELEASE(m_pBuffer);
		}
	};

	/// VertexBuffer NXp̋L|C^NX
	class VertexBufferPtr : public SharedPtr<VertexBuffer>
	{
	public:
		VertexBufferPtr() : SharedPtr<VertexBuffer>() {}
		explicit VertexBufferPtr(VertexBuffer* rep) : SharedPtr<VertexBuffer>(rep) {}
		VertexBufferPtr(const VertexBufferPtr &r) : SharedPtr<VertexBuffer>(r) {}
	protected:
		void destory(void)
		{
			SharedPtr<VertexBuffer>::destroy();
		}
	};

	class VertexStream
	{
		friend GraphicsDevice;

	private:
		GraphicsDevice* m_pDevice;
		UINT mStreamNumber;

		VertexStream(UINT streamNumber, GraphicsDevice* device)
		{
			mStreamNumber = streamNumber;
			m_pDevice = device;
		}

	public:
		void SetSource(const VertexBuffer* vb, int offsetInBytes, int vertexStride)
		{
			m_pDevice->_getD3D9Device()->SetStreamSource(mStreamNumber, vb->_getBuffer(), offsetInBytes, vertexStride);
		}

		void SetSource(const VertexBufferPtr &vb, int offsetInBytes, int vertexStride)
		{
			m_pDevice->_getD3D9Device()->SetStreamSource(mStreamNumber, vb->_getBuffer(), offsetInBytes, vertexStride);
		}
	};
}

#endif
