#include "BuddhaGraphicsDeviceManager.h"
#include "BuddhaGame.h"

namespace BuddhaXNA
{
	GraphicsDeviceManager::GraphicsDeviceManager(Game *game)
		: m_pGame(game),
		  m_pDevice(NULL),
		  mIsFullScreen(false)
	{
		game->GetServices()->AddService(String(L"GraphicsDeviceManager"), static_cast<void*>(this));
	}
	//------------------------------------------------------------------------
	GraphicsDeviceManager::~GraphicsDeviceManager()
	{
		SAFE_DELETE(m_pDevice);
	}
	//------------------------------------------------------------------------
	void GraphicsDeviceManager::CreateDevice(void)
	{
		D3DPRESENT_PARAMETERS d3dpp;
		ZeroMemory(&d3dpp, sizeof(d3dpp));

		d3dpp.BackBufferWidth = m_pGame->GetWindow()->GetClientBounds().Width;
		d3dpp.BackBufferHeight = m_pGame->GetWindow()->GetClientBounds().Height;
		d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;
		d3dpp.BackBufferCount = 1;
		d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
		d3dpp.MultiSampleQuality = 0;
		d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
		d3dpp.hDeviceWindow = m_pGame->GetWindow()->GetHandle();
		d3dpp.Windowed = mIsFullScreen ? FALSE : TRUE;
		d3dpp.EnableAutoDepthStencil = TRUE;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		d3dpp.Flags = 0;
		d3dpp.FullScreen_RefreshRateInHz = 0;
		d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

		m_pDevice = new GraphicsDevice(D3DDEVTYPE_HAL, m_pGame->GetWindow()->GetHandle(),
			D3DCREATE_MIXED_VERTEXPROCESSING, d3dpp);
	}
}