#include "BuddhaMath.h"
#include "BuddhaVector3.h"

namespace BuddhaXNA
{
	const Real MathHelper::Pi = Real(4.0 * atan(1.0));
	const Real MathHelper::TwoPi = Real(8.0 * atan(1.0));
	//------------------------------------------------------------------------
	bool Math::RealEqual(Real a, Real b, Real tolerance)
	{
		return (fabs(b - a) <= tolerance);
	}
	//------------------------------------------------------------------------
	Quaternion MathHelper::CreateQuaternionRotationTo(const Vector3 &src, const Vector3 &dest)
	{
		return CreateQuaternionRotationTo(src, dest, Vector3::Zero);
	}
	//------------------------------------------------------------------------
	Quaternion MathHelper::CreateQuaternionRotationTo(const Vector3 &src, const Vector3 &dest,
		const Vector3& fallbackAxis)
	{
		Quaternion q;

		// K
		Vector3 v0 = src;
		Vector3 v1 = dest;
		v0.Normalize();
		v1.Normalize();

		FLOAT d = Vector3::Dot(v0, v1);

		if (d >= 1.0f)
		{
			return Quaternion::Identity;
		}

		Real s = Math::Sqrt((1.0f + d) * 2);

		if (s < 1e-6f)
		{
			if (fallbackAxis != Vector3::Zero)
			{
				// 180 x]
				q = Quaternion::CreateFromAxisAngle(fallbackAxis, MathHelper::Pi);
			}
			else
			{
				// ]쐬
				Vector3 axis = Vector3::Cross(Vector3::UnitX, src);
				if (axis.DistanceSquared() < (1e-06 * 1e-06))
					axis = Vector3::Cross(Vector3::UnitY, src);

				axis.Normalize();

				q = Quaternion::CreateFromAxisAngle(axis, MathHelper::Pi);
			}
		}
		else
		{
			Real invs = 1 / s;

			Vector3 c = Vector3::Cross(v0, v1);

			q.X = c.X * invs;
			q.X = c.Y * invs;
			q.Z = c.Z * invs;
			q.W = (FLOAT)(s * 0.5);

			q.Normalize();
		}

		return q;
	}
}

