#include "BuddhaQuaternion.h"
#include "BuddhaMath.h"
#include "BuddhaVector3.h"

namespace BuddhaXNA
{
	//------------------------------------------------------------------------
	const Quaternion Quaternion::Zero(0.0f, 0.0f, 0.0f, 0.0f);
	const Quaternion Quaternion::Identity(0.0f, 0.0f, 0.0f, 1.0f);
	//------------------------------------------------------------------------
	Quaternion::Quaternion()
	{
		W = X = Y = Z = 0.0f;
	}
	//------------------------------------------------------------------------
	Quaternion::Quaternion(FLOAT fX, FLOAT fY, FLOAT fZ, FLOAT fW)
	{
		X = fX;
		Y = fY;
		Z = fZ;
		W = fW;
	}
	//------------------------------------------------------------------------
	Quaternion Quaternion::CreateFromAxisAngle(const Vector3 &axis, const FLOAT angle)
	{
		Real halfAngle = 0.5 * angle;
		Real fSin = Math::Sin(halfAngle);

		return Quaternion(
				(FLOAT)(fSin * axis.X),
				(FLOAT)(fSin * axis.Y),
				(FLOAT)(fSin * axis.Z),
				(FLOAT)(Math::Cos(halfAngle))
			);
	}
	//------------------------------------------------------------------------
	void Quaternion::CreateFromAxisAngle(const Vector3 &axis, const FLOAT angle, Quaternion &out)
	{
		Real halfAngle = 0.5 * angle;
		Real fSin = Math::Sin(halfAngle);

		out.W = (FLOAT)(Math::Cos(halfAngle));
		out.X = (FLOAT)(fSin * axis.X);
		out.Y = (FLOAT)(fSin * axis.Y);
		out.Z = (FLOAT)(fSin * axis.Z);
	}
	//------------------------------------------------------------------------
	Quaternion Quaternion::Inverse(const Quaternion &q)
	{
		Real norm = q.W * q.W + q.X * q.X + q.Y * q.Y + q.Z * q.Z;
		if (norm > 0.0)
		{
			Real invNorm = 1.0 / norm;
			return Quaternion(
				(FLOAT)(-q.X * invNorm),
				(FLOAT)(-q.Y * invNorm),
				(FLOAT)(-q.Z * invNorm),
				(FLOAT)(q.W * invNorm)
			);
		}

		return Zero;
	}
	//------------------------------------------------------------------------
	void Quaternion::Normalize(void)
	{
		FLOAT length = X * X + Y * Y + Z * Z + W * W;
		Real factor = 1.0f / Math::Sqrt(length);

		X *= factor;
		Y *= factor;
		Z *= factor;
		W *= factor;
	}
}