#include "BuddhaTexture2D.h"
#include "BuddhaGraphicsDevice.h"
#include "BuddhaXNAPrerequisites.h"
#include "BuddhaUtils.h"

#include <d3dx9.h>

static VOID WINAPI __filltexture(
	D3DXVECTOR4* pOut, const D3DXVECTOR2* pTexCoord,
	const D3DXVECTOR2* pTexelSize, LPVOID pData)
{
	*pOut = *(D3DXVECTOR4*)pData;
}


namespace BuddhaXNA
{
	Texture2D::Texture2D()
	{
		m_pTexture = NULL;
	}
	//------------------------------------------------------------------------
	Texture2D::~Texture2D()
	{
		SAFE_RELEASE(m_pTexture);
	}
	//------------------------------------------------------------------------
	void Texture2D::FromFile(GraphicsDevice* device, LPCWSTR filePath, Texture2D** pOut)
	{
		*pOut = NULL;
		D3DXIMAGE_INFO info;

		String truePath = Utils::FindMediaFile(String(filePath));

		if (truePath == L"")
			throw "The file is not found.";

		HRESULT hr = D3DXGetImageInfoFromFile(truePath.c_str(), &info);
		if (FAILED(hr))
		{
			// TODO O
			return;
		}

		Texture2D* pTexture = new Texture2D();

		hr = D3DXCreateTextureFromFileEx(
			device->_getD3D9Device(),
			truePath.c_str(),
			info.Width,
			info.Height,
			D3DX_DEFAULT,
			0,
			D3DFMT_UNKNOWN,
			D3DPOOL_MANAGED,
			D3DX_DEFAULT,
			D3DX_DEFAULT,
			0,
			NULL,
			NULL,
			&pTexture->m_pTexture);

		if (FAILED(hr))
		{
			// ToDo {͗O
			delete pTexture;
			return;
		}

		Rectangle rect;

		rect.X = 0;
		rect.Y = 0;
		rect.Width = info.Width;
		rect.Height = info.Height;

		pTexture->_setRectangle(rect);

		*pOut = pTexture;
	}
	//------------------------------------------------------------------------
	Texture2DPtr Texture2D::FromFile(GraphicsDevice* device, LPCWSTR filePath)
	{
		Texture2D* pTex;
		FromFile(device, filePath, &pTex);

		return Texture2DPtr(pTex);
	}
	//------------------------------------------------------------------------
	Texture2D* Texture2D::CreateDummy(GraphicsDevice* device)
	{
		Texture2D* pTexture = new Texture2D();

		HRESULT hr;

		hr = D3DXCreateTexture(device->_getD3D9Device(), 1, 1, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &pTexture->m_pTexture);

		if (FAILED(hr))
		{
			delete pTexture;
			return NULL;
		}

		// hԂ
		D3DXVECTOR4 color(1, 1, 1, 1);
		D3DXFillTexture(pTexture->m_pTexture, __filltexture, &color);

		Rectangle rect;

		rect.X = 0;
		rect.Y = 0;
		rect.Width = 1;
		rect.Height = 1;

		pTexture->_setRectangle(rect);

		return pTexture;
	}
	//------------------------------------------------------------------------
	Texture2D::Texture2D(GraphicsDevice* device, int width, int height, int numberLevels, DWORD usage, D3DFORMAT format)
	{
		HRESULT hr;

		hr = D3DXCreateTexture(
			device->_getD3D9Device(),
			width, height, numberLevels,
			usage,
			format,
			D3DPOOL_DEFAULT,
			&m_pTexture
		);

		if (FAILED(hr))
			throw "Could not create texture.";

		Rectangle rect;
		rect.X = rect.Y = 0;
		rect.Width = width;
		rect.Height = height;

		_setRectangle(rect);
	}
}



