#include "BuddhaVertexDeclaration.h"
#include "BuddhaGraphicsDevice.h"
#include "BuddhaVector2.h"
#include "BuddhaVector3.h"

#include <d3dx9.h>

using namespace BuddhaXNA;

const D3DVERTEXELEMENT9 VertexPositionColor::VertexElements[] = 
{
	{ 0, 0,                 D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, sizeof(FLOAT) * 3, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
	D3DDECL_END()
};

const int VertexPositionColor::SizeInBytes = sizeof(FLOAT) * 7;

const D3DVERTEXELEMENT9 VertexPositionNormalTexture::VertexElements[] = 
{
	{ 0, 0,                 D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
	{ 0, sizeof(FLOAT) * 3, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	{ 0, sizeof(FLOAT) * 6, D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
	D3DDECL_END()
};

const int VertexPositionNormalTexture::SizeInBytes = sizeof(FLOAT) * 8;
//const int VertexPositionNormalTexture::SizeInBytes = sizeof(FLOAT) * 8;

const D3DVERTEXELEMENT9 VertexPositionNormalBlendColorTexture::VertexElements[] = 
{
	{ 0,  0, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,     0 },
	{ 0, 12, D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,       0 },
	{ 0, 24, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BLENDWEIGHT,  0 },
	{ 0, 40, D3DDECLTYPE_FLOAT4,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BLENDINDICES, 0 },
	{ 0, 56, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,        0 },
	{ 0, 60, D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,     0 },
	D3DDECL_END()
};

struct VERTEXPOSITIONNORMALBLENDCOLORTEXTURE
{
	D3DXVECTOR3 Position;
	D3DXVECTOR3 Normal;
	FLOAT Weight[4];
	D3DXVECTOR4 Index;
	D3DCOLOR Color;
	D3DXVECTOR2 Tex;
};

const int VertexPositionNormalBlendColorTexture::SizeInBytes = sizeof(VERTEXPOSITIONNORMALBLENDCOLORTEXTURE);

VertexDeclaration::VertexDeclaration(GraphicsDevice* graphicDevice, const D3DVERTEXELEMENT9* elements)
{
	m_pDevice = graphicDevice;
	m_pVertexDeclaration = NULL;

	m_pDevice->_getD3D9Device()->CreateVertexDeclaration(elements, &m_pVertexDeclaration);
}

VertexPositionColor::VertexPositionColor()
{
	Position = Vector3::Zero;
	Color = D3DCOLOR_XRGB(255, 255, 255);
}

VertexPositionNormalTexture::VertexPositionNormalTexture()
{
	Position = Vector3::Zero;
	Normal = Vector3::Zero;
	TextureCoordinate = Vector2::Zero;
}

VertexPositionNormalTexture::VertexPositionNormalTexture(const Vector3& position, const Vector3& normal, const Vector2& texcoord)
{
	Position = position;
	Normal = normal;
	TextureCoordinate = texcoord;
}
